%  This code creates an image segmentation problem and demonstrates how to
%  use the code 'pdhg_segment.m' to solve it.
%  The segmentation problem is.
%            min_u    TV(u) + mu <u, (im-c1)^2-(im-c2)^2 >
%  where 'im' denotes the image, and 'c1'/'c2' denote the intensity inside
%  the foreground/background regions.
%  This File requires that 'pdhg_adaptive.m' and 'pdhg_segment.m' be in 
%  your current path. 
%
%  For an explanation of PDHG and how it applies to this problem, see
%    "Adaptive Primal-Dual Hybrid Gradient Methods for Saddle-Point
%    Problems"  available at <http://arxiv.org/abs/1305.0546>
%

% create test image
im = phantom(128,128);

% Scale parameter for the segmentation
mu = 1000;

% scale image from 0-1
im = im/max(max(im));

%  The mean of each region that we wish to segment
c1 = .3;
c2 = .1;

%  Compute the segmentation
[seg, out] = pdhg_segment( im, mu, c1, c2);


%% plot results
figure('Position', [100, 50, 350, 700]);
% show original image
subplot(3,1,1);
imagesc(im);
title('Image');
% The segmentation
subplot(3,1,2);
imagesc(seg>0.5);
title('Segmentation');
%  Plot the residuals, which measure convergence
subplot(3,1,3);
semilogy(out.p,'b');
hold on;
semilogy(out.d,'r');
hold off;
legend('Primal Residual','Dual Residual');
title('Convergence Curves');
