%  This code creates an image denoising problem, and demonstrates how to
%  use the code 'pdhg_tv' to solve it using adaptive PDHG.
%  The TV denoising problem is.
%            min_u    TV(u) + mu/2 || u - noisy ||^2
%  where TV(*) denoted the total-variation semi-norm, ||*|| denotes the
%  l2 (frobenius) norm, and 'noisy' denotes a 2D noisy pixel array (image).
%  This File requires that 'pdhg_adaptive.m' and 'pdhg_tv.m' be in 
%  your current path. 
%
%  For an explanation of PDHG and how it applies to this problem, see
%    "Adaptive Primal-Dual Hybrid Gradient Methods for Saddle-Point
%    Problems"  available at <http://arxiv.org/abs/1305.0546>
%

%% Create noisy image and setup problem
im  = phantom(128);  %  The test image
mu = 0.01;             % 0.1  Denoising parameter

%  Scale the image from 0-255.  
im = im*255/max(max(im)); 
noisy = im + 10*randn(size(im));  % add noise

%%  Solve the problem 
[denoised, out] = pdhg_tv(noisy, mu);

%% plot results
% show original image
figure;
subplot(2,2,1);
imagesc(im);
title('Original');
% show noisy iamge
subplot(2,2,2);
imagesc(noisy);
title('Noisy');
% show denoised image
subplot(2,2,3);
imagesc(denoised);
title('Denoised');
%  Plot the residuals, which measure convergence
subplot(2,2,4);
semilogy(out.p,'b');
hold on;
semilogy(out.d,'r');
hold off;
legend('Primal Residual','Dual Residual');
title('Convergence Curves');
